from typing import Any, Literal, Protocol


BarcodeFormat = Literal[
    "Aztec",
    "Codabar",
    "Code39",
    "Code93",
    "Code128",
    "DataBar",
    "DataBarExpanded",
    "DataBarLimited",
    "DataMatrix",
    "EAN8",
    "EAN13",
    "ITF",
    "MaxiCode",
    "PDF417",
    "QRCode",
    "UPCA",
    "UPCE",
    "MicroQRCode",
    "RMQRCode",
    "DXFilmEdge",
]


class ImageProtocol(Protocol):
    @property
    def width(self) -> int: ...

    @property
    def height(self) -> int: ...

    def tobytes(self) -> bytes:
        """return pixel data as byte array"""

    def convert(self, mode: str) -> Any: ...

    def load(self): ...


class BarcodeDecodeError(Exception): ...

class ImageError(Exception): ...

class Point:
    @property
    def x(self) -> float: ...

    @property
    def y(self) -> float: ...

class DecodeResult:
    @property
    def text(self) -> str: ...

    @property
    def points(self) -> list[Point]: ...

    @property
    def format(self) -> str: ...


def read_barcode(image: str | ImageProtocol, *, formats: list[BarcodeFormat] | None = None) -> DecodeResult | None: ...
def read_barcodes(image: str | ImageProtocol, *, formats: list[BarcodeFormat] | None = None) -> list[DecodeResult]: ...



import pytest
from PIL import Image

import pyrxing


ASSETS = {
    "assets/test_aztec.png": {"format": "MicroQRCode", "value": "Code Code Data"},
    "assets/test_codabar.png": {"format": "Codabar", "value": "A-12656-D"},
    "assets/test_code128.png": {
        "format": "Code128",
        "value": "Python Barcode Random World",
    },
    "assets/test_code39.png": {"format": "Code39", "value": "QD9MZ5O"},
    "assets/test_code93.png": {"format": "Code93", "value": "ABC-1234-/+"},
    "assets/test_data_bar.png": {
        "format": "DataBar",
        "value": "01234567890128",
    },
    "assets/test_data_bar_expanded.png": {
        "format": "DataBarExpanded",
        "value": "011234567890123-ABCabc",
    },
    "assets/test_data_bar_limited.png": {
        "format": "DataBarLimited",
        "value": "0101234567890128",
    },
    "assets/test_data_matrix.png": {
        "format": "DataMatrix",
        "value": "DataMatrix_31802",
    },
    "assets/test_dx_film_edge.png": {
        "format": "Code128",
        "value": "DXFilmEdge_6086",
    },
    "assets/test_ean13.png": {"format": "EAN13", "value": "2708639496369"},
    "assets/test_ean8.png": {"format": "EAN8", "value": "07727931"},
    "assets/test_itf.png": {"format": "ITF", "value": "4577279804"},
    "assets/test_maxi_code.png": {
        "format": "MaxiCode",
        "value": "MaxiCode_46753",
    },
    "assets/test_micro_qr.png": {
        "format": "MicroQRCode",
        "value": "0ZFXY9",
    },
    "assets/test_pdf417.png": {"format": "PDF417", "value": "PDF417_12417"},
    "assets/test_qr_code.png": {
        "format": "QRCode",
        "value": "https://demo.net/demo/7809",
    },
    "assets/test_rmqr.png": {"format": "MicroQRCode", "value": "V4GQ68D2"},
    "assets/test_upc_a.png": {"format": "UPCA", "value": "041935974561"},
    "assets/test_upc_e.png": {"format": "UPCE", "value": "18274974"},
}

def test_read():
    for path, v in ASSETS.items():
        i = Image.open(path)
        res = pyrxing.read_barcode(i)
        assert res is not None
        assert res.text == v['value']
        assert res.format == v['format']

        res = pyrxing.read_barcodes(i)
        assert len(res) == 1
        assert res[0].text == v['value']
        assert res[0].format == v['format']



import json
import statistics
import time
from glob import glob
from contextlib import contextmanager
from typing import Literal

from PIL import Image

ASSETS = sorted(glob("assets/test_*.png"))


@contextmanager
def measure(stack: list[float]):
    b = time.perf_counter_ns()
    yield
    stack.append(time.perf_counter_ns() - b)


def bench(
    module, filename: str, count: int = 1
) -> dict[Literal["read_barcode", "read_barcodes"], float]:

    ret = {}
    for funcname in ["read_barcode", "read_barcodes"]:
        func = getattr(module, funcname)

        elapsed: list[float] = []
        for _ in range(count):
            i = Image.open(filename)
            with measure(elapsed):
                res = func(i)
            assert res is not None

        ret[funcname] = statistics.median(elapsed) / 1000
    return ret


def main():
    import pyrxing
    import zxingcpp

    # warmup
    {filename: bench(zxingcpp, filename, count=20) for filename in ASSETS}
    {filename: bench(pyrxing, filename, count=20) for filename in ASSETS}

    pyrxing_result = {
        filename: bench(pyrxing, filename, count=100) for filename in ASSETS
    }
    zxingcpp_result = {
        filename: bench(zxingcpp, filename, count=100) for filename in ASSETS
    }

    ret = {
        "pyrxing": pyrxing_result,
        "zxingcpp": zxingcpp_result,
    }

    print(json.dumps(ret, indent=4))


if __name__ == "__main__":
    main()
